from abstra_cli.resources.resources import Resource
import abstra_cli.messages as messages
import abstra_cli.utils as utils
import abstra_cli.apis as apis


class Vars(Resource):
    @staticmethod
    def list():
        vars = apis.list_workspace_vars()
        messages.print_vars(vars)

    @staticmethod
    def add(*args, **kwargs):
        vars = list(args)
        file = kwargs.get("f") or kwargs.get("file")
        if file:
            with open(file, "r") as f:
                vars.extend([v for v in f.read().split("\n") if v])

        processed_vars = []
        processed_names = []
        for var in vars:
            name, value = utils.parse_env_var(var)
            if not name or not value:
                messages.invalid_variable(var)
                return False
            if name in processed_names:
                messages.duplicate_variable(var)
                return False
            processed_vars.append({"name": name, "value": value})

        added_vars = apis.add_workspace_vars(processed_vars)
        messages.print_vars(added_vars)
        print(f"\nAdded {len(added_vars)} enviroment variables")

    @staticmethod
    def remove(*args, **kwargs):
        deleted_vars = apis.delete_workspace_vars(args)
        messages.print_vars(deleted_vars)
        print(f"\nDeleted {len(deleted_vars)} vars")



import urllib.request
import urllib.response
import abstra_cli.apis.main as api_main


def upload_file(filepath, file):
    response_json = api_main.hf_api_runner("POST", "put-url", {"filepath": filepath})
    req = urllib.request.Request(
        url=response_json["putURL"], method="PUT", data=file.read()
    )
    res = urllib.request.urlopen(req)
    return res.status < 400


def get_file_signed_url(filepath):
    response_json = api_main.hf_api_runner("POST", "get-url", {"filepath": filepath})
    return response_json.get("getURL")


def list_workspace_files():
    return api_main.hf_api_runner("GET", "files")


def delete_file(filepath):
    return api_main.hf_api_runner("DELETE", "file", {"filepath": filepath})



import json
import requests
import urllib.request
import urllib.response

from .public import get_info_from_token
import abstra_cli.credentials as credentials


HACKERFORMS_API_URL = "https://hackerforms-api.abstra.cloud"
HACKERFORMS_HASURA_URL = "https://hackerforms-hasura.abstra.cloud/v1/graphql"
ABSTRA_ASSETS_UPLOAD_URL = "https://upload.abstra.cloud"


def get_auth_info():
    api_token = credentials.get_credentials()
    if not api_token:
        return None, None, None
    workspace_id, author_id = get_info_from_token(api_token)
    return api_token, workspace_id, author_id


def asset_upload(filepath, file):
    response = requests.request(
        "POST",
        f"{ABSTRA_ASSETS_UPLOAD_URL}/asset/",
        headers={
            "cache-control": "no-cache",
            "Pragma": "no-cache",
            "content-type": "application/json",
        },
        data=json.dumps({"filepath": filepath}),
    )

    response_json = response.json()
    try:

        req = urllib.request.Request(
            url=response_json["putURL"], method="PUT", data=file
        )
        res = urllib.request.urlopen(req)
        if res.status < 400:
            return response_json["getURL"]
    except Exception as e:
        print(e)

    raise Exception("Some error ocurred in asset upload")


def hf_api_runner(method, path, data=None):
    api_token, workspace_id, _ = get_auth_info()
    response = requests.request(
        method,
        f"{HACKERFORMS_API_URL}/workspaces/{workspace_id}/{path}",
        data=json.dumps(data) if data else None,
        headers={"content-type": "application/json", "API-Authorization": api_token},
    )
    return response.json()


def hf_hasura_runner(query, variables={}):
    api_token = credentials.get_credentials()
    response = requests.post(
        HACKERFORMS_HASURA_URL,
        data=json.dumps({"query": query, "variables": variables}),
        headers={"content-type": "application/json", "API-Authorization": api_token},
    )
    if response.status_code >= 300:
        raise Exception(f"Request error: {response.text}")
    jsond = response.json()

    if "data" in jsond:
        return jsond["data"]

    raise Exception(jsond["errors"])


def get_subdomain():
    query = """
        query Subdomains {
            subdomains {
                name
            }
        }
    """

    subdomains = hf_hasura_runner(query, {}).get("subdomains", [])
    if not len(subdomains):
        print("Could not find subdomain.")
        exit()

    return subdomains[0]["name"]



from progress.bar import FillingSquaresBar

import abstra_cli.utils as utils


def read_credentials():
    print(
        "Abstra API Tokens can be found in your workspace or at https://forms.abstra.run/generate-token."
    )
    credentials = input(f"API Token: ")
    if not credentials:
        raise Exception("No API token configured")
    return credentials


def show_progress(message, max) -> FillingSquaresBar:
    return FillingSquaresBar(message, suffix="%(percent)d%%", max=max)


def print_files(files):
    files.sort(key=lambda x: x["Key"])
    max_d = utils.digits(max([f["Size"] for f in files]))
    for file in files:
        print(
            f"{utils.format_digits(file['Size'], max_d)} - {file['LastModified']}: {file['Key']}"
        )


def print_vars(vars):
    vars.sort(key=lambda x: x["name"])
    for var in vars:
        print(f"{var['name']}={var['value']}")


def print_packages(packages):
    packages.sort(key=lambda x: x["name"])
    for pkg in packages:
        print(f"{pkg['name']}{'==' + pkg['version'] if pkg['version'] else ''}")


def print_forms(forms):
    forms.sort(key=lambda x: x["path"])
    max_path = max([len(f["path"]) for f in forms])
    max_title = max([len(f["title"]) for f in forms])
    for form in forms:
        enabled = "enabled" if form["script"]["enabled"] else "disabled"
        print(
            f"{utils.format_digits(form['path'], max_path)} - {utils.format_digits(form['title'], max_title)} ({enabled})"
        )


def print_hooks(hooks):
    hooks.sort(key=lambda x: x["path"])
    max_path = max([len(f["path"]) for f in hooks])
    max_title = max([len(f["title"]) for f in hooks])
    for form in hooks:
        enabled = "enabled" if form["script"]["enabled"] else "disabled"
        print(
            f"{utils.format_digits(form['path'], max_path)} - {utils.format_digits(form['title'], max_title)} ({enabled})"
        )


def print_jobs(jobs):
    jobs.sort(key=lambda x: x["identifier"])
    max_idt = max([len(f["identifier"]) for f in jobs])
    max_title = max([len(f["title"]) for f in jobs])
    max_schedule = max([len(f["schedule"]) for f in jobs])
    for j in jobs:
        enabled = "enabled" if j["script"]["enabled"] else "disabled"
        print(
            f"{utils.format_digits(j['identifier'], max_idt)} - {utils.format_digits(j['schedule'], max_schedule)} - {utils.format_digits(j['title'], max_title)} ({enabled})"
        )


def created_message(resource: str, form: str):
    print(f"{resource} created successfully: {form}")


def upserted_message(resource: str, form: str):
    print(f"{resource} upserted successfully: {form}")


def create_failed(resource: str):
    print(f"Failed to create {resource}")


def updated_message(resource: str, form: str):
    print(f"{resource} updated successfully: {form}")


def deleted_message(resource: str, form: str):
    print(f"{resource} deleted successfully: {form}")


def update_failed(resource: str, form: str):
    print(f"Failed to update {resource}: {form}")


def not_implemented(*args, **kwargs):
    print("Invalid command")


def required_argument(argument):
    print(f"required argument: [{argument}]")


def required_parameter(parameter):
    print(f"required parameter: --{parameter} [{parameter}]")


def invalid_variable(argument):
    print(f"invalid variable: {argument}")


def invalid_parameter(parameter):
    print(f"invalid parameter: --{parameter} [{parameter}]")


def invalid_flag_parameter_value(parameter):
    print(
        f"invalid parameter value: --{parameter} [{parameter}] value must be true or false"
    )


def invalid_non_flag_parameter_value(parameter):
    print(f"invalid parameter value: --{parameter} [{parameter}] value must be string")


def invalid_background_parameter_value():
    print(
        f"""
        invalid parameter value: --background [background] value must be a color or image path.\n
        Allowed image extensions are: [.png, .jpeg, .jpg, .svg]"""
    )


def duplicate_variable(argument):
    print(f"duplicate variable: {argument}")


def code_and_file_not_allowed():
    print("you can set either code or file parameters, but not both.")


def missing_parameters_to_update(form):
    print(f"missing parameters to be updated of form {form}")


def file_path_does_not_exists_message(path):
    print(f"file path not found: {path}")


def upsert_without_identifier(idt):
    print(f"Failed to upsert without identifier: {idt}")


def error_upload_background_message(path):
    print(f"Some error happened during background upload: {path}")


def form_url(url):
    print(f"Opening URL {url}")


def hook_url(url, method):
    print(f"Making {method} request to URL {url}")



import os
import re
import pathlib
from setuptools import setup

regex = "^v(\d+\.\d+\.\d+)$"
TAG = os.getenv("TAG", "v0.0.0")
if not TAG or not re.search(regex, TAG):
    raise ValueError("TAG enviroment variable must be in the format v1.2.3")
version = re.search(regex, TAG).group(1)

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# The list of requirements
requirements = (HERE / "requirements.txt").read_text().split("\n")

setup(
    name="abstra-cli",
    version=version,
    description="Abstra CLI",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/abstra-app/abstra-cli",
    license="MIT",
    packages=[
        "abstra_cli",
        "abstra_cli.resources",
        "abstra_cli.apis",
        "abstra_cli.utils",
    ],
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "abstra=abstra_cli.cli:main",
            "abstra-cli=abstra_cli.cli:main",
        ],
    },
)
